@@ -7,7 +7,6 @@
  */
 
 // Class for reading in, checking and storing user parameters
-
 #include "parameters.h"
 
 parameters::parameters(int argc, char *argv[])
@@ -19,21 +18,24 @@ parameters::parameters(int argc, char *argv[])
 		this->argv.push_back(tmp);
 	}
 
+	bcf_format = false;
 	BED_exclude = false;
 	BED_file = "";
-	chr_to_keep = "";
-	chr_to_exclude = "";
+	derived = false;
 	diff_discordance_matrix = false;
 	diff_file = "";
+	diff_file_bcf = false;
 	diff_file_compressed = false;
 	diff_indv_discordance = false;
+	diff_indv_map_file = "";
 	diff_site_discordance = false;
 	diff_switch_error = false;
 	end_pos = numeric_limits<int>::max();
 	exclude_positions_file = "";
 	force_write_index = false;
-	hapmap_fst_file = "";
-	hapmap_fst_file_compressed = false;
+	fst_window_size = -1;
+	fst_window_step = -1;
+	gatk = false;
 	indv_exclude_file = "";
 	indv_keep_file = "";
 	invert_mask = false;
@@ -41,6 +43,8 @@ parameters::parameters(int argc, char *argv[])
 	recode_all_INFO = false;
 	ld_bp_window_size = numeric_limits<int>::max();
 	ld_snp_window_size = numeric_limits<int>::max();
+	ld_bp_window_min = -1;
+	ld_snp_window_min = -1;
 	min_mac = -1;
 	min_maf = -1.0;
 	mask_file = "";
@@ -72,18 +76,22 @@ parameters::parameters(int argc, char *argv[])
 	output_as_IMPUTE = false;
 	output_as_ldhat_phased = false;
 	output_as_ldhat_unphased = false;
-	output_BEAGLE_genotype_likelihoods = false;
+	output_BEAGLE_genotype_likelihoods_GL = false;
+	output_BEAGLE_genotype_likelihoods_PL = false;
 	output_counts = false;
 	output_filter_summary = false;
 	output_filtered_sites = false;
 	output_freq = false;
 	output_geno_depth = false;
+	output_geno_chisq = false;
 	output_geno_rsq = false;
 	output_hap_rsq = false;
 	output_het = false;
 	output_HWE = false;
+	output_indel_hist = false;
 	output_indv_depth = false;
-	output_interchromosomal_rsq = false;
+	output_interchromosomal_hap_rsq = false;
+	output_interchromosomal_geno_rsq = false;
 	output_LROH = false;
 	output_missingness = false;
 	output_N_PCA_SNP_loadings = -1;
@@ -103,11 +111,14 @@ parameters::parameters(int argc, char *argv[])
 	phased_only = false;
 	PCA_no_normalisation = false;
 	pi_window_size = 0;
+	pi_window_step = 0;
 	plink_output = false;
 	plink_tped_output = false;
 	positions_file = "";
 	recode = false;
+	recode_bcf = false;
 	recode_to_stream = false;
+	recode_bcf_to_stream = false;
 	remove_all_filtered_genotypes = false;
 	remove_all_filtered_sites = false;
 	remove_indels = false;
@@ -127,18 +138,23 @@ void parameters::read_parameters()
 	{
 		in_str = argv[i];
 		if (in_str == "--vcf") { vcf_filename = get_arg(i+1); vcf_compressed = false; i++; } 				// VCF file to process
+		else if (in_str == "--bcf") { vcf_filename = get_arg(i+1); bcf_format = true; vcf_compressed = false; i++; } 				// BCF file to process
 		else if (in_str == "--012") output_012_matrix = true;							// Output as 0/1/2 matrix
-		else if (in_str == "--BEAGLE-GL") { output_BEAGLE_genotype_likelihoods = true; min_alleles=2; max_alleles=2; }	// Output as BEAGLE Genotype Likelihood format
+		else if (in_str == "--BEAGLE-GL") { output_BEAGLE_genotype_likelihoods_GL = true; min_alleles=2; max_alleles=2; }	// Output as BEAGLE Genotype Likelihood format
+		else if (in_str == "--BEAGLE-PL") { output_BEAGLE_genotype_likelihoods_PL = true; min_alleles=2; max_alleles=2; }	// Output as BEAGLE Genotype Likelihood format
 		else if (in_str == "--bed") { BED_file = get_arg(i+1); i++; BED_exclude=false; }
-		else if (in_str == "--chr") { chr_to_keep = get_arg(i+1); i++; }					// Chromosome to process
+		else if (in_str == "--chr") { chrs_to_keep.insert(get_arg(i+1)); i++; }					// Chromosome to process
 		else if (in_str == "--counts") output_counts = true;								// Output per-site allele count statistics
 		else if (in_str == "--counts2") {output_counts = true; suppress_allele_output = true; }								// Output per-site allele count statistics
 		else if (in_str == "--depth") output_indv_depth = true;							// Output per-individual coverage statistics
+		else if (in_str == "--derived") derived = true;								// Output frequencies so that AA is first.
 		else if (in_str == "--diff-discordance-matrix") { diff_discordance_matrix = true; }	// Calculate some concensus statistics
 		else if (in_str == "--diff-indv-discordance") { diff_indv_discordance = true; }	// Calculate some concensus statistics
+		else if (in_str == "--diff-indv-map") { diff_indv_map_file = get_arg(i+1); i++; }	// Map individual ids in the second file
 		else if (in_str == "--diff-site-discordance") { diff_site_discordance = true; }	// Calculate some concensus statistics
 		else if (in_str == "--diff-switch-error") { diff_switch_error = true; }	// Calculate some concensus statistics
 		else if (in_str == "--diff") { diff_file = get_arg(i+1); diff_file_compressed = false; i++; }	// Calculate some concensus statistics
+		else if (in_str == "--diff-bcf") { diff_file = get_arg(i+1); diff_file_bcf = true; i++; }	// Calculate some concensus statistics
 		else if (in_str == "--exclude-bed") { BED_file = get_arg(i+1); i++; BED_exclude=true; }
 		else if (in_str == "--exclude") { snps_to_exclude_file = get_arg(i+1); i++; }				// List of SNPs to exclude
 		else if (in_str == "--exclude-positions") { exclude_positions_file = get_arg(i+1); i++; }
@@ -149,22 +165,28 @@ void parameters::read_parameters()
 		else if (in_str == "--freq") output_freq = true;									// Output per-site frequency statistics
 		else if (in_str == "--freq2") {output_freq = true;	suppress_allele_output = true; }							// Output per-site frequency statistics
 		else if (in_str == "--from-bp") { start_pos = atoi(get_arg(i+1).c_str()); i++; }					// Start position
+		else if (in_str == "--fst-window-size") { fst_window_size = atoi(get_arg(i+1).c_str()); i++; }                  // Window size for Fst calculation
+		else if (in_str == "--fst-window-step") { fst_window_step = atoi(get_arg(i+1).c_str()); i++; }                  // Window step for Fst calculation
+		else if (in_str == "--gatk") gatk = true;								//Denote BCF file came from GATK
 		else if (in_str == "--geno-depth") output_geno_depth = true;						// Output Depth for each genoptype
 		else if (in_str == "--geno-r2") { output_geno_rsq = true; min_alleles = 2; max_alleles = 2; } // Output pairwise LD (r^2)
+		else if (in_str == "--geno-chisq") { output_geno_chisq = true; } // Output pairwise LD (r^2)
+		else if (in_str == "--geno-r2-positions") { geno_rsq_position_list = get_arg(i+1); i++; min_alleles = 2; max_alleles = 2; }	// Output pairwise LD (r^2)
 		else if (in_str == "--geno") { min_site_call_rate = atof(get_arg(i+1).c_str()); i++; }				// Minimum per-site call rate
 		else if (in_str == "--get-INFO") { INFO_to_extract.push_back(get_arg(i+1)); i++; }	// Add to list of INFO fields to extract
 		else if (in_str == "--gzdiff") { diff_file = get_arg(i+1); diff_file_compressed = true; i++; }	// Calculate some concensus statistics
 		else if (in_str == "--gzvcf") { vcf_filename = get_arg(i+1); vcf_compressed = true; i++; } 				// VCF file to process
 		else if (in_str == "--hap-r2") { output_hap_rsq = true; phased_only = true; min_alleles = 2; max_alleles = 2; }	// Output pairwise LD (r^2)
-		else if (in_str == "--hapmap-fst-file") { hapmap_fst_file = get_arg(i+1); hapmap_fst_file_compressed = false; i++; }
+		else if (in_str == "--hap-r2-positions") { hap_rsq_position_list = get_arg(i+1); i++; phased_only = true; min_alleles = 2; max_alleles = 2; }	// Output pairwise LD (r^2)
 		else if (in_str == "--hapmap-fst-pop") { hapmap_fst_populations.push_back(get_arg(i+1)); i++; }
-		else if (in_str == "--hapmap-fst-gzfile") { hapmap_fst_file = get_arg(i+1); hapmap_fst_file_compressed = true; i++; }
 		else if (in_str == "--hardy") output_HWE = true;									// Output HWE statistics
 		else if (in_str == "--het") output_het = true;									// Output heterozygosity statistics
+		else if (in_str == "--hist-indel-len") output_indel_hist = true;
 		else if (in_str == "--hwe") { max_alleles = 2; min_HWE_pvalue = atof(get_arg(i+1).c_str()); i++; }					// Minimum per-site HWE p-value
 		else if (in_str == "--IMPUTE") { output_as_IMPUTE = true; phased_only=true; min_site_call_rate=1.0; min_alleles=2; max_alleles=2; }// Output as IMPUTE format
 		else if (in_str == "--indv") { indv_to_keep.insert(get_arg(i+1)); i++; }						// List of individuals to keep
-		else if (in_str == "--interchrom-geno-r2") { output_interchromosomal_rsq = true; min_alleles = 2; max_alleles = 2;	}	// Output pairwise LD (r^2)
+		else if (in_str == "--interchrom-hap-r2") { output_interchromosomal_hap_rsq = true; phased_only = true;  min_alleles = 2; max_alleles = 2;	}	// Output pairwise LD (r^2)
+		else if (in_str == "--interchrom-geno-r2") { output_interchromosomal_geno_rsq = true; min_alleles = 2; max_alleles = 2;	}	// Output pairwise LD (r^2)
 		else if (in_str == "--invert-mask") { mask_file = get_arg(i+1); i++; invert_mask = true; }
 		else if (in_str == "--keep-filtered") { site_filter_flags_to_keep.insert(get_arg(i+1)); i++; }	// Remove a specific filter flag
 		else if (in_str == "--keep") { indv_keep_file = get_arg(i+1); i++; }						// List of individuals to keep
@@ -173,6 +195,8 @@ void parameters::read_parameters()
 		else if (in_str == "--keep-INFO-all") { recode_all_INFO=true; }	// Old command (soon to be depreciated)
 		else if (in_str == "--ld-window-bp") { ld_bp_window_size = atoi(get_arg(i+1).c_str()); i++; }	// Max bp distance for LD output
 		else if (in_str == "--ld-window") { ld_snp_window_size = atoi(get_arg(i+1).c_str()); i++; }		// Max SNP distance for LD output
+		else if (in_str == "--ld-window-bp-min") { ld_bp_window_min = atoi(get_arg(i+1).c_str()); i++; }	// Max bp distance for LD output
+		else if (in_str == "--ld-window-min") { ld_snp_window_min = atoi(get_arg(i+1).c_str()); i++; }		// Max SNP distance for LD output
 		else if (in_str == "--ldhat-geno") { output_as_ldhat_unphased = true; }
 		else if (in_str == "--ldhat") { output_as_ldhat_phased = true; phased_only = true; } // Output as LDhat format
 		else if (in_str == "--LROH") {output_LROH = true;}
@@ -201,7 +225,7 @@ void parameters::read_parameters()
 		else if (in_str == "--missing") output_missingness = true;						// Output Individual and Site missingness summaries
 		else if (in_str == "--non-ref-ac") { min_non_ref_ac = atoi(get_arg(i+1).c_str()); i++; }				// Minimum Site non-ref AC
 		else if (in_str == "--non-ref-af") { min_non_ref_af = atof(get_arg(i+1).c_str()); i++; }				// Minimum Site non-ref AF
-		else if (in_str == "--not-chr") { chr_to_exclude = get_arg(i+1); i++; }					// Chromosome to process
+		else if (in_str == "--not-chr") { chrs_to_exclude.insert(get_arg(i+1)); i++; }					// Chromosome to process
 		else if (in_str == "--out") { output_prefix = get_arg(i+1); i++; }							// Output file prefix
 		else if (in_str == "--pca") { output_PCA = true; min_alleles=2; max_alleles=2; }
 		else if (in_str == "--pca-no-norm") { output_PCA = true; PCA_no_normalisation = true; min_alleles=2; max_alleles=2; }
@@ -211,6 +235,8 @@ void parameters::read_parameters()
 		else if (in_str == "--plink-tped") plink_tped_output = true;	// Output as PLINK tped file
 		else if (in_str == "--positions") { positions_file = get_arg(i+1); i++; }
 		else if (in_str == "--recode") recode = true;										// Output VCF file
+		else if (in_str == "--recode-bcf") recode_bcf = true;										// Output VCF file
+		else if (in_str == "--recode-bcf-to-stream") {  recode_bcf_to_stream = true; LOG.set_screen_output(false); }
 		else if (in_str == "--recode-to-stream") {  recode_to_stream = true; LOG.set_screen_output(false); }
 		else if (in_str == "--recode-INFO-all") { recode_all_INFO=true; }		// Specify INFO to keep when recoding
 		else if (in_str == "--recode-INFO") { recode_INFO_to_keep.insert(get_arg(i+1)); i++; }		// Specify INFO to keep when recoding
@@ -239,6 +265,7 @@ void parameters::read_parameters()
 		else if (in_str == "--TsTv-by-qual") {output_TsTv_by_qual = true; }
 		else if (in_str == "--weir-fst-pop") { weir_fst_populations.push_back(get_arg(i+1)); i++; }
 		else if (in_str == "--window-pi") { pi_window_size = atoi(get_arg(i+1).c_str()); i++; }
+		else if (in_str == "--window-pi-step") { pi_window_step = atoi(get_arg(i+1).c_str()); i++; }
 		else
 			error("Unknown option: " + string(in_str), 0);
 		i++;
@@ -260,27 +287,40 @@ void parameters::print_params()
 	LOG.printLOG("Parameters as interpreted:\n");
 	if (vcf_filename != defaults.vcf_filename)
 	{
-		if (vcf_compressed == false)
+		if (bcf_format == true)
+			LOG.printLOG("\t--bcf " + vcf_filename + "\n");
+		else if (vcf_compressed == false)
 			LOG.printLOG("\t--vcf " + vcf_filename + "\n");
 		else
 			LOG.printLOG("\t--gzvcf " + vcf_filename + "\n");
 	}
 
-	if (chr_to_exclude != defaults.chr_to_exclude) LOG.printLOG("\t--not-chr " + chr_to_exclude + "\n");
-	if (chr_to_keep != defaults.chr_to_keep) LOG.printLOG("\t--chr " + chr_to_keep + "\n");
+	if (chrs_to_keep.size() > 0)
+	{
+		for (set<string>::iterator it=chrs_to_keep.begin(); it != chrs_to_keep.end(); ++it)
+		{
+			string tmp = *it;
+			LOG.printLOG("\t--chr " + tmp + "\n");
+		}
+	}
+
+	if (chrs_to_exclude.size() > 0)
+	{
+		for (set<string>::iterator it=chrs_to_exclude.begin(); it != chrs_to_exclude.end(); ++it)
+		{
+			string tmp = *it;
+			LOG.printLOG("\t--not-chr " + tmp + "\n");
+		}
+	}
+
+	if (derived != defaults.derived) LOG.printLOG("\t--derived\n");
 	if (end_pos != defaults.end_pos) LOG.printLOG("\t--to-bp " + output_log::int2str(end_pos) + "\n");
 	if (exclude_positions_file != defaults.exclude_positions_file) LOG.printLOG("\t--exclude-positions " + exclude_positions_file + "\n");
 	if (force_write_index != defaults.force_write_index) LOG.printLOG("\t--force-index-write\n");
 	if (FORMAT_id_to_extract != defaults.FORMAT_id_to_extract) LOG.printLOG("\t--extract-FORMAT-info " + FORMAT_id_to_extract + "\n");
-	if (hapmap_fst_file != defaults.hapmap_fst_file)
-	{
-		if (hapmap_fst_file_compressed == false)
-			LOG.printLOG("\t--hapmap-fst-file " + hapmap_fst_file + "\n");
-		else
-		{
-			LOG.printLOG("\t--hapmap-fst-gzfile " + hapmap_fst_file + "\n");
-		}
-	}
+	if (gatk != defaults.gatk) LOG.printLOG("\t--gatk\n");
+	if (geno_rsq_position_list != defaults.geno_rsq_position_list) LOG.printLOG("\t--geno-r2-positions " + geno_rsq_position_list + "\n");
+	if (hap_rsq_position_list != defaults.hap_rsq_position_list) LOG.printLOG("\t--hap-r2-positions " + hap_rsq_position_list + "\n");
 	if (hapmap_fst_populations.size() != 0)
 	{
 		for (unsigned int ui=0; ui<hapmap_fst_populations.size(); ui++)
@@ -297,6 +337,8 @@ void parameters::print_params()
 	if (keep_only_indels != defaults.keep_only_indels) LOG.printLOG("\t--keep-only-indels\n");
 	if (ld_bp_window_size != defaults.ld_bp_window_size) LOG.printLOG("\t--ld-window-bp " + output_log::int2str(ld_bp_window_size) + "\n");
 	if (ld_snp_window_size != defaults.ld_snp_window_size) LOG.printLOG("\t--ld-window " + output_log::int2str(ld_snp_window_size) + "\n");
+	if (ld_bp_window_min != defaults.ld_bp_window_min) LOG.printLOG("\t--ld-window-bp-min " + output_log::int2str(ld_bp_window_min) + "\n");
+	if (ld_snp_window_min != defaults.ld_snp_window_min) LOG.printLOG("\t--ld-window-min " + output_log::int2str(ld_snp_window_min) + "\n");
 	if (min_mac != defaults.min_mac) LOG.printLOG("\t--mac " + output_log::dbl2str(min_mac, 3) + "\n");
 	if (min_maf != defaults.min_maf) LOG.printLOG("\t--maf " + output_log::dbl2str(min_maf, 3) + "\n");
 	if (max_alleles != defaults.max_alleles) LOG.printLOG("\t--max-alleles " + output_log::int2str(max_alleles) + "\n");
@@ -325,7 +367,8 @@ void parameters::print_params()
 	if (min_non_ref_af != defaults.min_non_ref_af) LOG.printLOG("\t--non-ref-af " + output_log::dbl2str(min_non_ref_af, 3) + "\n");
 	if (output_012_matrix) LOG.printLOG("\t--012\n");
 	if (output_as_IMPUTE) LOG.printLOG("\t--IMPUTE\n");
-	if (output_BEAGLE_genotype_likelihoods) LOG.printLOG("\t--BEAGLE-GL\n");
+	if (output_BEAGLE_genotype_likelihoods_GL) LOG.printLOG("\t--BEAGLE-GL\n");
+	if (output_BEAGLE_genotype_likelihoods_PL) LOG.printLOG("\t--BEAGLE-PL\n");
 	if (output_counts && (suppress_allele_output==false)) LOG.printLOG("\t--counts\n");
 	if (output_counts && (suppress_allele_output==true)) LOG.printLOG("\t--counts2\n");
 	if (output_filter_summary) LOG.printLOG("\t--FILTER-summary\n");
@@ -334,11 +377,14 @@ void parameters::print_params()
 	if (output_freq && (suppress_allele_output==true)) LOG.printLOG("\t--freq2\n");
 	if (output_geno_depth) LOG.printLOG("\t--geno-depth\n");
 	if (output_geno_rsq) LOG.printLOG("\t--geno-r2\n");
+	if (output_geno_chisq) LOG.printLOG("\t--geno-chisq\n");
 	if (output_hap_rsq) LOG.printLOG("\t--hap-r2\n");
 	if (output_het) LOG.printLOG("\t--het\n");
 	if (output_HWE) LOG.printLOG("\t--hardy\n");
+	if (output_indel_hist != defaults.output_indel_hist) LOG.printLOG("\t--hist-indel-len \n");
 	if (output_indv_depth) LOG.printLOG("\t--depth\n");
-	if (output_interchromosomal_rsq) LOG.printLOG("\t--interchrom-geno-r2\n");
+	if (output_interchromosomal_hap_rsq) LOG.printLOG("\t--interchrom-hap-r2\n");
+	if (output_interchromosomal_geno_rsq) LOG.printLOG("\t--interchrom-geno-r2\n");
 	if (output_LROH != defaults.output_LROH) LOG.printLOG("\t--LROH\n");
 	if (output_missingness) LOG.printLOG("\t--missing\n");
 	if (output_PCA != defaults.output_PCA)
@@ -364,11 +410,14 @@ void parameters::print_params()
 	if (output_TsTv_by_qual) LOG.printLOG("\t--TsTv-by-qual\n");
 	if (phased_only) LOG.printLOG("\t--phased\n");
 	if (pi_window_size != defaults.pi_window_size) LOG.printLOG("\t--window-pi " + output_log::int2str(pi_window_size) + "\n");
+	if (pi_window_step != defaults.pi_window_step) LOG.printLOG("\t--window-pi-step " + output_log::int2str(pi_window_step) + "\n");
 	if (plink_output) LOG.printLOG("\t--plink\n");
 	if (plink_tped_output) LOG.printLOG("\t--plink-tped\n");
 	if (positions_file != defaults.positions_file) LOG.printLOG("\t--positions " + positions_file + "\n");
 	if (recode) LOG.printLOG("\t--recode\n");
+	if (recode_bcf) LOG.printLOG("\t--recode-bcf\n");
 	if (recode_to_stream) LOG.printLOG("\t--recode-to-stream\n");
+	if (recode_bcf_to_stream) LOG.printLOG("\t--recode-bcf-to-stream\n");
 	if (remove_all_filtered_genotypes) LOG.printLOG("\t--remove-filtered-geno-all\n");
 	if (remove_all_filtered_sites) LOG.printLOG("\t--remove-filtered-all\n");
 	if (remove_indels != defaults.remove_indels) LOG.printLOG("\t--remove-indels\n");
@@ -407,7 +456,9 @@ void parameters::print_params()
 
 	if (diff_file != defaults.diff_file)
 	{
-		if (vcf_compressed == false)
+		if ( diff_file_bcf == true )
+			LOG.printLOG("\t--diff-bcf " + diff_file + "\n");
+		else if (diff_file_compressed == false)
 			LOG.printLOG("\t--diff " + diff_file + "\n");
 		else
 			LOG.printLOG("\t--gzdiff " + diff_file + "\n");
@@ -415,6 +466,7 @@ void parameters::print_params()
 		if (diff_indv_discordance == true) LOG.printLOG("\t--diff-indv-discordance\n");
 		if (diff_discordance_matrix == true) LOG.printLOG("\t--diff-discordance-matrix\n");
 		if (diff_switch_error == true) LOG.printLOG("\t--diff-switch-error\n");
+		if (diff_indv_map_file != defaults.diff_indv_map_file) LOG.printLOG("\t--diff-indv-map " + diff_indv_map_file + "\n");
 	}
 
 	if (recode_INFO_to_keep.size() > 0)
@@ -510,13 +562,12 @@ void parameters::print_help()
 	}
 }
 
-// TODO: Rewrite this function to do some proper error checking.
 void parameters::check_parameters()
 {
 	parameters defaults(0, 0);
 	if (vcf_filename == "") error("VCF required.", 0);
 	if (end_pos < start_pos) error("End position must be greater than Start position.", 1);
-	if (((end_pos != numeric_limits<int>::max()) || (start_pos != -1)) && (chr_to_keep == "")) error("Require a chromosome when specifying a range.", 2);
+	if (((end_pos != numeric_limits<int>::max()) || (start_pos != -1)) && (chrs_to_keep.size() != 1)) error("Require a single chromosome when specifying a range.", 2);
 	if (max_maf < min_maf) error("Maximum MAF must be not be less than Minimum MAF.", 4);
 	if (max_mac < min_mac) error("Maximum MAC must be not be less than Minimum MAC.", 4);
 	if (min_maf != defaults.min_maf)
@@ -538,11 +589,11 @@ void parameters::check_parameters()
 	if (max_mean_depth < min_mean_depth) error("Max Mean Depth must be greater the Min Mean Depth.", 7);
 	if (max_indv_mean_depth < min_indv_mean_depth) error("Max Indv Mean Depth must be greater the Min Indv Mean Depth.", 8);
 	if (max_genotype_depth < min_genotype_depth) error("Max Genotype Depth must be greater than Min Genotype Depth.", 9);
-	if (((output_as_ldhat_phased == true) || (output_as_ldhat_unphased)) && (chr_to_keep == "")) error("Require a chromosome (--chr) when outputting LDhat format.", 11);
-	if ((output_BEAGLE_genotype_likelihoods == true) && (chr_to_keep == "")) error("Require a chromosome (--chr) when outputting Beagle likelihoods.", 11);
+	if (((output_as_ldhat_phased == true) || (output_as_ldhat_unphased)) && (chrs_to_keep.size() != 1)) error("Require a chromosome (--chr) when outputting LDhat format.", 11);
+	if ((output_BEAGLE_genotype_likelihoods_GL == true) && (chrs_to_keep.size() != 1)) error("Require a chromosome (--chr) when outputting Beagle likelihoods.", 11);
+	if ((output_BEAGLE_genotype_likelihoods_PL == true) && (chrs_to_keep.size() != 1)) error("Require a chromosome (--chr) when outputting Beagle likelihoods.", 11);
 	if (min_kept_mask_value > 9) error("Min Mask value must be between 0 and 9.", 14);
-	if ((output_LROH == true) && (chr_to_keep == "")) error("Require a chromosome (--chr) when outputting LROH.", 11);
-	if ((chr_to_keep != "") && (chr_to_exclude != "") && (chr_to_keep == chr_to_exclude)) error("Chromosome to keep cannot match chromosome to exclude.",15);
+	if ((output_LROH == true) && (chrs_to_keep.size() != 1)) error("Require a chromosome (--chr) when outputting LROH.", 11);
 	if (output_TsTv_bin_size < 0) error("TsTv bin size must be > 0",16);
 	if (output_Tajima_D_bin_size < 0) error("Tajima D bin size must be > 0", 17);
 	if (pi_window_size < 0) error("Pi Window size must be > 0", 18);
