@@ -10,6 +10,7 @@
 #ifndef PARAMETERS_H_
 #define PARAMETERS_H_
 
+#include <algorithm>
 #include <cstdio>
 #include <cstdlib>
 #include <iostream>
@@ -24,19 +25,23 @@ extern output_log LOG;
 
 using namespace std;
 
-const string VCFTOOLS_VERSION="v0.1.9.0";
+const string VCFTOOLS_VERSION="v0.1.11";
 
 class parameters
 {
 public:
+	bool bcf_format;
 	bool BED_exclude;
 	string BED_file;
-	string chr_to_exclude;
-	string chr_to_keep;
+	set<string> chrs_to_exclude;
+	set<string> chrs_to_keep;
+	bool derived;
 	bool diff_discordance_matrix;
 	string diff_file;
+	bool diff_file_bcf;
 	bool diff_file_compressed;
 	bool diff_indv_discordance;
+	string diff_indv_map_file;
 	bool diff_site_discordance;
 	bool diff_switch_error;
 	int end_pos;
@@ -44,10 +49,13 @@ public:
 	bool force_write_index;
 	string FORMAT_id_to_extract;
 	set<string> geno_filter_flags_to_exclude;
-	string hapmap_fst_file;
-	bool hapmap_fst_file_compressed;
+	string geno_rsq_position_list;
+	string hap_rsq_position_list;
 	vector<string> hapmap_fst_populations;
 	vector<string> weir_fst_populations;
+	int fst_window_size;
+	int fst_window_step;
+	bool gatk;
 	string indv_exclude_file;
 	string indv_keep_file;
 	set<string> indv_to_exclude;
@@ -57,6 +65,8 @@ public:
 	bool keep_only_indels;
 	int ld_bp_window_size;
 	int ld_snp_window_size;
+	int ld_bp_window_min;
+	int ld_snp_window_min;
 	int min_mac;
 	double min_maf;
 	string mask_file;
@@ -88,18 +98,22 @@ public:
 	bool output_as_IMPUTE;
 	bool output_as_ldhat_phased;
 	bool output_as_ldhat_unphased;
-	bool output_BEAGLE_genotype_likelihoods;
+	bool output_BEAGLE_genotype_likelihoods_GL;
+	bool output_BEAGLE_genotype_likelihoods_PL;
 	bool output_counts;
 	bool output_filter_summary;
 	bool output_filtered_sites;
 	bool output_freq;
 	bool output_geno_depth;
+	bool output_geno_chisq;
 	bool output_geno_rsq;
 	bool output_hap_rsq;
 	bool output_het;
 	bool output_HWE;
+	bool output_indel_hist;
 	bool output_indv_depth;
-	bool output_interchromosomal_rsq;
+	bool output_interchromosomal_hap_rsq;
+	bool output_interchromosomal_geno_rsq;
 	bool output_LROH;
 	bool output_missingness;
 	int output_N_PCA_SNP_loadings;
@@ -119,11 +133,14 @@ public:
 	bool phased_only;
 	bool PCA_no_normalisation;
 	int pi_window_size;
+	int pi_window_step;
 	bool plink_output;
 	bool plink_tped_output;
 	string positions_file;
 	bool recode;
+	bool recode_bcf;
 	bool recode_to_stream;
+	bool recode_bcf_to_stream;
 	set<string> recode_INFO_to_keep;
 	bool recode_all_INFO;
 	bool remove_all_filtered_genotypes;
