@@ -7,26 +7,7 @@
  */
 
 #include "vcf_entry.h"
-
-void vcf_entry::set_CHROM(const string &in)
-{
-	CHROM = in;
-}
-
-void vcf_entry::set_POS(const int in)
-{
-	POS = in;
-}
-
-void vcf_entry::set_ID(const string &in)
-{
-	ID = in;
-}
-
-void vcf_entry::set_REF(const string &in)
-{
-	REF = in;
-}
+#include "entry.h"
 
 void vcf_entry::set_ALT(const string &in)
 {
@@ -41,32 +22,11 @@ void vcf_entry::set_ALT(const string &in)
 	parsed_ALT = true;
 }
 
-void vcf_entry::add_ALT_allele(const string &in)
-{
-	if (in != ".")
-	{
-		if (find(ALT.begin(), ALT.end(),in) == ALT.end())
-		{
-			ALT.push_back(in);
-		}
-	}
-	parsed_ALT = true;
-}
-
 void vcf_entry::set_QUAL(const double in)
 {
 	QUAL = in;
 }
 
-void vcf_entry::add_FILTER_entry(const string &in)
-{
-	if ((in != "PASS") && (in != "."))
-		if (find(FILTER.begin(), FILTER.end(), in) == FILTER.end())
-			FILTER.push_back(in);
-	sort(FILTER.begin(), FILTER.end());
-	parsed_FILTER = true;
-}
-
 void vcf_entry::set_FORMAT(const string &in)
 {
 	FORMAT.resize(0);
@@ -147,6 +107,8 @@ void vcf_entry::set_indv_GENOTYPE_and_PHASE(unsigned int indv, const string &in)
 
 void vcf_entry::set_indv_GENOTYPE_and_PHASE(unsigned int indv, const pair<int, int> &genotype, char phase)
 {
+	ploidy.resize(N_indv);
+	ploidy[indv] = 2;
 	set_indv_GENOTYPE_ids(indv, genotype);
 	set_indv_PHASE(indv, phase);
 	parsed_GT[indv] = true;
@@ -154,6 +116,8 @@ void vcf_entry::set_indv_GENOTYPE_and_PHASE(unsigned int indv, const pair<int, i
 
 void vcf_entry::set_indv_GENOTYPE_and_PHASE(unsigned int indv, const pair<string, string> &genotype, char phase)
 {
+	ploidy.resize(N_indv);
+	ploidy[indv] = 2;
 	set_indv_GENOTYPE_alleles(indv, genotype);
 	set_indv_PHASE(indv, phase);
 	parsed_GT[indv] = true;
@@ -214,7 +178,11 @@ void vcf_entry::set_indv_GQUALITY(unsigned int indv, double in)
 {
 	parsed_GQ[indv] = true;
 	if (in == -1)
+	{
+		if (GQUALITY.size() > 0)
+			GQUALITY[indv] = -1;
 		return;
+	}
 	if (GQUALITY.size() == 0)
 		GQUALITY.resize(N_indv, -1);
 
@@ -227,7 +195,11 @@ void vcf_entry::set_indv_DEPTH(unsigned int indv, int in)
 {
 	parsed_DP[indv] = true;
 	if (in == -1)
+	{
+		if (DEPTH.size() > 0)
+			DEPTH[indv] = -1;
 		return;
+	}
 	if (DEPTH.size() == 0)
 		DEPTH.resize(N_indv, -1);
 
@@ -248,13 +220,13 @@ void vcf_entry::add_indv_GFILTER(unsigned int indv, const string &in)
 void vcf_entry::set_indv_GFILTER(unsigned int indv, const string &in)
 {
 	parsed_FT[indv] = true;
-	if ((in.size() == 0) || (in == "."))
-		return;
 
 	if (GFILTER.size() == 0)
 		GFILTER.resize(N_indv);
 
 	GFILTER[indv].resize(0);
+	if ((in.size() == 0) || (in == "."))
+		return;
 
 	static istringstream ss;
 	static string ith_FILTER;
@@ -323,7 +295,7 @@ void vcf_entry::set_INFO(const string &INFO_str)
 	parsed_INFO = true;
 }
 
-void vcf_entry::add_INFO_descriptor(const string &in)
+int vcf_entry::add_INFO_descriptor(const string &in, unsigned int index)
 {
 	size_t found=in.find("##INFO=");
 	if (found!=string::npos)
@@ -380,11 +352,31 @@ void vcf_entry::add_INFO_descriptor(const string &in)
 		}
 		else LOG.error("Expected Description entry as fourth field in INFO description: " + in);
 
-		INFO_map[I.ID] = I;
+		if ( FORMAT_reverse_map.find( I.ID ) != FORMAT_reverse_map.end() )
+		{
+			INFO_map[ I.ID ] = I;
+			INFO_reverse_map[I.ID] = FORMAT_reverse_map[I.ID];
+			return 0;
+		}
+		else if ( FILTER_reverse_map.find( I.ID ) != FILTER_reverse_map.end() )
+		{
+			INFO_map[ I.ID ] = I;
+			INFO_reverse_map[I.ID] = FILTER_reverse_map[ I.ID ];
+			return 0;
+		}
+		else if (INFO_map.find(I.ID) != INFO_map.end())
+			return 0;
+		else
+		{
+			INFO_map[I.ID] = I;
+			INFO_reverse_map[I.ID] = index;
+			return 1;
+		}
 	}
+	return 0;
 }
 
-void vcf_entry::add_FILTER_descriptor(const string &in)
+int vcf_entry::add_FILTER_descriptor(const string &in, unsigned int index)
 {
 	size_t found=in.find("##FILTER=");
 	if (found!=string::npos)
@@ -416,11 +408,31 @@ void vcf_entry::add_FILTER_descriptor(const string &in)
 		else
 			LOG.error("Expected Description as second field in FILTER description: " + in);
 
-		FILTER_map[ID] = Description;
+		if ( INFO_reverse_map.find( ID ) != INFO_reverse_map.end() )
+		{
+			FILTER_map[ ID ] = Description;
+			FILTER_reverse_map[ID] = INFO_reverse_map[ ID ];
+			return 0;
+		}
+		else if ( FORMAT_reverse_map.find( ID ) != FORMAT_reverse_map.end() )
+		{
+			FILTER_map[ ID ] = Description;
+			FILTER_reverse_map[ID] = FORMAT_reverse_map[ID];
+			return 0;
+		}
+		else if (FILTER_map.find(ID) != FILTER_map.end())
+			return 0;
+		else
+		{
+			FILTER_map[ID] = Description;
+			FILTER_reverse_map[ID] = index;
+			return 1;
+		}
 	}
+	return 0;
 }
 
-void vcf_entry::add_FORMAT_descriptor(const string &in)
+int vcf_entry::add_FORMAT_descriptor(const string &in, unsigned int index)
 {
 	size_t found=in.find("##FORMAT=");
 	if (found!=string::npos)
@@ -477,6 +489,55 @@ void vcf_entry::add_FORMAT_descriptor(const string &in)
 		}
 		else LOG.error("Expected Description entry as fourth field in FORMAT description: " + in);
 
-		FORMAT_map[I.ID] = I;
+		if ( FILTER_reverse_map.find( I.ID ) != FILTER_reverse_map.end() )
+		{
+			FORMAT_map[ I.ID ] = I;
+			FORMAT_reverse_map[I.ID] = FILTER_reverse_map[ I.ID ];
+			return 0;
+		}
+		else if ( INFO_reverse_map.find( I.ID ) != INFO_reverse_map.end() )
+		{
+			FORMAT_map[ I.ID ] = I;
+			FORMAT_reverse_map[I.ID] = INFO_reverse_map[ I.ID ];
+			return 0;
+		}
+		else if (FORMAT_map.find(I.ID) != FORMAT_map.end())
+			return 0;
+		else
+		{
+			FORMAT_map[I.ID] = I;
+			FORMAT_reverse_map[I.ID] = index;
+			return 1;
+		}
 	}
+	return 0;
+}
+
+void vcf_entry::add_CONTIG_descriptor(const string &in, unsigned int index)
+{
+	size_t found_start=in.find_first_of("<");
+	size_t found_end=in.find_last_of(">");
+	string details = in.substr(found_start+1, found_end-found_start-1);
+
+	vector<string> tokens;
+	entry::tokenize(details, ',', tokens);
+	Field_description I;
+	bool id_found = false;
+	vector<string> entry;
+
+	for (unsigned int ui=0; ui<tokens.size(); ui++)
+	{
+		entry::tokenize(tokens[ui], '=', entry);
+		if (entry[0] == "ID")
+		{
+			I.ID = entry[1];
+			id_found = true;
+		}
+		else if (entry[0] == "length") I.Length = entry[1];
+		else if (entry[0] == "assembly") I.Assembly = entry[1];
+	}
+	if (id_found == false)
+		LOG.warning("CONTIG declaration found without ID: "+ in + "\n");
+
+	CONTIG_map[I.ID] = index;
 }
