@@ -1,24 +1,34 @@
 /*
  * vcf_file.cpp
  *
- *  Created on: Aug 19, 2009
- *      Author: Adam Auton
- *      ($Revision: 230 $)
+ *  Created on: Dec 11, 2012
+ *      Author: amarcketta
  */
 
 #include "vcf_file.h"
 
-vcf_file::vcf_file(const string &filename, bool compressed, const string &chr, const string &exclude_chr, bool force_write_index) :
-	filename(filename),
-	compressed(compressed),
-	has_body(false),
-	has_file_format(false),
-	has_genotypes(false),
-	has_header(false),
-	has_meta(false)
+vcf_file::vcf_file(const string &fname, bool comp, const set<string> &chrs_to_keep, const set<string> &exclude_chrs, bool force_write_index)
 {
+	filename = fname; compressed = comp;
+	has_body = false; has_file_format = false;
+	has_header = false; has_meta = false;
+	bcf_format = false; has_genotypes = false;
+	has_contigs = false; contig_index = 0;
+	gzMAX_LINE_LEN = 0; meta.clear();
+
 	open();
-	scan_file(chr, exclude_chr, force_write_index);
+	scan_file(chrs_to_keep, exclude_chrs, force_write_index);
+}
+
+vcf_file::vcf_file()
+{
+	gzvcf_in = false;
+	gz_readbuffer = NULL;
+	compressed = false;	has_body = false;
+	has_file_format = false; has_header = false;
+	has_meta = false; has_genotypes = false;
+	has_contigs = false; contig_index = 0;
+	gzMAX_LINE_LEN = 0; meta.clear();
 }
 
 vcf_file::~vcf_file()
@@ -27,7 +37,7 @@ vcf_file::~vcf_file()
 }
 
 // Parse VCF meta information
-void vcf_file::parse_meta(const string &line)
+void vcf_file::parse_meta(const string &line, unsigned int &line_index)
 {
 	has_meta = true;
 	meta.push_back(line);
@@ -44,23 +54,35 @@ void vcf_file::parse_meta(const string &line)
 	found=line.find("##INFO=");
 	if (found!=string::npos)
 	{	// Found an INFO descriptor
-		vcf_entry::add_INFO_descriptor(line);
+		line_index += vcf_entry::add_INFO_descriptor(line, line_index);
 	}
 
 	found=line.find("##FILTER=");
 	if (found!=string::npos)
 	{	// Found a FILTER descriptor
-		vcf_entry::add_FILTER_descriptor(line);
+		line_index += vcf_entry::add_FILTER_descriptor(line, line_index);
 	}
 
 	found=line.find("##FORMAT=");
 	if (found!=string::npos)
 	{	// Found a genotype filter descriptor
-		vcf_entry::add_FORMAT_descriptor(line);
+		line_index += vcf_entry::add_FORMAT_descriptor(line, line_index);
+	}
+
+//ALT FIELDS NO LONGER COUNT
+//	found=line.find("##ALT=");
+//	if (found!=string::npos)
+//		line_index += 1;
+
+	found=line.find("##contig=");
+	if (found!=string::npos)
+	{	// Found a contig descriptor
+		vcf_entry::add_CONTIG_descriptor(line, contig_index);
+		contig_index++;
+		has_contigs = true;
 	}
 }
 
-// Parse VCF header, and extract individuals etc.
 void vcf_file::parse_header(const string &line)
 {
 	// #CHROM	POS	ID	REF	ALT	QUAL	FILTER	INFO	(FORMAT	NA00001 NA00002 ... )
@@ -75,7 +97,6 @@ void vcf_file::parse_header(const string &line)
 	has_genotypes = false;
 	while (!header.eof())
 	{
-		//header >> tmp_str;
 		getline(header, tmp_str, '\t');
 		switch (count)
 		{
@@ -110,22 +131,23 @@ void vcf_file::parse_header(const string &line)
 		LOG.warning("FORMAT field without genotypes?");
 }
 
-
-// Read VCF file
-void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool force_write_index)
+void vcf_file::scan_file(const set<string> &chrs_to_keep, const set<string> &exclude_chrs, bool force_write_index)
 {
-	bool filter_by_chr = (chr != "");
-	bool exclude_by_chr = (exclude_chr != "");
+	bool filter_by_chr = (chrs_to_keep.size() != 0);
+	bool exclude_by_chr = (exclude_chrs.size() != 0);
 	string index_filename = filename + ".vcfidx";
 	bool could_read_index_file = false;
 	if (force_write_index == false)
 		could_read_index_file = read_index_file(index_filename);
 	string CHROM, last_CHROM="";
-	int POS, last_POS = -1;
-	bool found_header = false;
-	bool found_meta = false;
+	unsigned int meta_counter = 1;
+
 	if (could_read_index_file == false)
 	{
+		int POS, last_POS = -1;
+		bool found_header = false;
+		bool found_meta = false;
+
 		LOG.printLOG("Building new index file.\n");
 		string line, CHROM, last_CHROM = "";
 		streampos filepos;
@@ -133,7 +155,7 @@ void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool forc
 		N_entries=0;
 		N_indv = 0;
 
-		while (!feof())
+		while (!eof())
 		{
 			filepos = get_filepos();
 			c = peek();
@@ -151,7 +173,7 @@ void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool forc
 				read_line(line);
 				if (line[1] == '#')
 				{	// Meta information
-					parse_meta(line);
+					parse_meta(line, meta_counter);
 					found_meta = true;
 				}
 				else
@@ -166,13 +188,23 @@ void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool forc
 					LOG.error("No header or meta information. Invalid file: " + filename);
 
 				read_CHROM_and_POS_and_skip_remainder_of_line(CHROM, POS);
+
+				if (POS == last_POS)
+				{
+					if (last_CHROM == CHROM)
+						LOG.one_off_warning("\tWarning - file contains entries with the same position. These entries will be processed separately.\n");
+				}
+				else if (last_POS > POS)
+				{
+					if (last_CHROM == CHROM)
+						LOG.error(" VCF file is not sorted at position " + CHROM + ":" + LOG.int2str(POS) + ".\n");
+				}
+
 				if (last_CHROM != CHROM)
 				{
 					LOG.printLOG("\tScanning Chromosome: " + CHROM + "\n");
 					last_CHROM = CHROM;
 				}
-				if (POS == last_POS)
-					LOG.one_off_warning("\tWarning - file contains entries with the same position. These entries will be processed separately.\n");
 
 				last_POS = POS;
 				entry_file_locations.push_back(filepos);
@@ -185,31 +217,37 @@ void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool forc
 
 		write_index_file(index_filename);
 	}
+	else
+	{
+		vector<string> meta_lines = meta; meta.resize(0);
+		meta_counter = 1;
+		for (unsigned int ui=0; ui<meta_lines.size(); ui++)
+			parse_meta(meta_lines[ui], meta_counter);
+	}
+
+	if (has_contigs == false)
+	{
+		vector<string> contig_vector;
+		get_default_contigs(contig_vector);
+		for(unsigned int ui=0; ui<contig_vector.size(); ui++)
+			vcf_entry::add_CONTIG_descriptor(contig_vector[ui], ui);
+	}
 
 	LOG.printLOG("File contains " + output_log::int2str(N_entries) + " entries and " + output_log::int2str(N_indv) + " individuals.\n");
-	vector<string> meta_lines = meta; meta.resize(0);
-	for (unsigned int ui=0; ui<meta_lines.size(); ui++)
-		parse_meta(meta_lines[ui]);
 	has_genotypes = (N_indv > 0);
 
-	bool already_found_required_chr = false;
-	bool already_filtered_required_chr = false;
 	if ((exclude_by_chr == true) || (filter_by_chr == true))
 	{
+		unsigned int N_found_required_chr = chrs_to_keep.size();
 		LOG.printLOG("Filtering by chromosome.\n");
 		for (unsigned int ui=0; ui<N_entries; ui++)
 		{
-			if (already_found_required_chr == true)
+			if ((filter_by_chr == true) && (N_found_required_chr == 0))
 			{
 				LOG.printLOG("Skipping Remainder.\n");
 				entry_file_locations.erase(entry_file_locations.begin()+ui, entry_file_locations.end());
 				break;
 			}
-			if (already_filtered_required_chr == true)
-			{
-				LOG.printLOG("Skipping Remainder.\n");
-				break;
-			}
 
 			set_filepos(entry_file_locations[ui]);
 			read_CHROM_only(CHROM);
@@ -217,20 +255,17 @@ void vcf_file::scan_file(const string &chr, const string &exclude_chr, bool forc
 			if (last_CHROM != CHROM)
 			{
 				LOG.printLOG("\tChromosome: " + CHROM + "\n");
-				if ((filter_by_chr == true) && (last_CHROM == chr))
-					already_found_required_chr = true;
-
-				if ((exclude_by_chr == true) && (last_CHROM == exclude_chr))
-					already_filtered_required_chr = true;
+				if ((filter_by_chr == true) && (chrs_to_keep.find(last_CHROM) != chrs_to_keep.end()))
+					N_found_required_chr--;
 
 				last_CHROM = CHROM;
 			}
-			if ((exclude_by_chr == true) && (CHROM == exclude_chr))
+			if ((exclude_by_chr == true) && (exclude_chrs.find(CHROM) != exclude_chrs.end()))
 			{
 				entry_file_locations[ui] = -1;
 				continue;
 			}
-			if ((filter_by_chr == true) && (CHROM != chr))
+			if ((filter_by_chr == true) && (chrs_to_keep.find(CHROM) == chrs_to_keep.end()))
 			{
 				entry_file_locations[ui] = -1;
 				continue;
@@ -265,17 +300,19 @@ void vcf_file::print(ostream &out, const set<string> &INFO_to_keep, bool keep_al
 			out << "\t" << indv[ui];
 	out << endl;
 
-	string vcf_line;
+	vector<char> variant_line;
+	entry * e = new vcf_entry(N_indv);
 	for (unsigned int s=0; s<N_entries; s++)
 		if (include_entry[s] == true)
 		{
-			get_vcf_entry(s, vcf_line);
-			vcf_entry e(N_indv, vcf_line);
-			e.parse_basic_entry(true, true, true);
-			e.parse_full_entry(true);
-			e.parse_genotype_entries(true,true,true,true);
-			e.print(out, INFO_to_keep, keep_all_INFO, include_indv, include_genotype[s]);
+			get_entry(s, variant_line);
+			e->reset(variant_line);
+			e->parse_basic_entry(true, true, true);
+			e->parse_full_entry(true);
+			e->parse_genotype_entries(true,true,true,true);
+			e->print(out, INFO_to_keep, keep_all_INFO, include_indv, include_genotype[s]);
 		}
+	delete e;
 }
 
 void vcf_file::print(const string &output_file_prefix, const set<string> &INFO_to_keep, bool keep_all_INFO)
@@ -293,53 +330,108 @@ void vcf_file::print(const string &output_file_prefix, const set<string> &INFO_t
 	LOG.printLOG("Done\n");
 }
 
-// Return the number of individuals that have not been filtered out
-int vcf_file::N_kept_individuals() const
+void vcf_file::print_bcf(BGZF* out, const set<string> &INFO_to_keep, bool keep_all_INFO)
 {
-	int N_kept = 0;
-	for (unsigned int ui=0; ui<include_indv.size(); ui++)
-		if (include_indv[ui] == true)
-			N_kept++;
-	return N_kept;
+	string header_str;
+	uint32_t len_text = 0;
+	vector<char> header;
+
+	char magic[5] = {'B','C','F','\2', '\1'};
+	bgzf_write(out, magic, 5);
+
+	for (unsigned int ui=0; ui<meta.size(); ui++)
+	{
+		for (unsigned int uj=0; uj<meta[ui].length(); uj++)
+			header.push_back( meta[ui][uj] );
+		header.push_back('\n');
+	}
+
+	if (has_contigs == false)
+	{
+		vector<string> contig_vector;
+		get_default_contigs(contig_vector);
+		for(unsigned int ui=0; ui<contig_vector.size(); ui++)
+		{
+			for(unsigned int uj=0; uj<contig_vector[ui].size(); uj++)
+				header.push_back(contig_vector[ui][uj]);
+			header.push_back('\n');
+		}
+	}
+
+	header_str = "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO";
+	if (N_indv > 0)
+		header_str += "\tFORMAT";
+
+	for (unsigned int ui=0; ui<N_indv; ui++)
+		if (include_indv[ui])
+		{
+			header_str += "\t";
+			header_str += indv[ui];
+		}
+	header_str += "\n";
+
+	for (unsigned int ui=0; ui<header_str.length(); ui++)
+		header.push_back( header_str[ui] );
+
+	header.push_back( '\0' );
+	len_text = header.size();
+
+	bgzf_write(out, (char *)&len_text, sizeof(len_text) );
+	bgzf_write(out, (char *)&header[0], len_text );
+
+	vector<char> variant_line;
+	entry * e = new vcf_entry(N_indv);
+	for (unsigned int s=0; s<N_entries; s++)
+		if (include_entry[s] == true)
+		{
+			get_entry(s, variant_line);
+			e->reset(variant_line);
+			e->parse_basic_entry(true, true, true);
+			e->parse_full_entry(true);
+			e->parse_genotype_entries(true,true,true,true);
+			e->print_bcf(out, INFO_to_keep, keep_all_INFO, include_indv, include_genotype[s]);
+		}
+	delete e;
 }
 
-// Return the number of sites that have not been filtered out
-int vcf_file::N_kept_sites() const
+void vcf_file::print_bcf(const string &output_file_prefix, const set<string> &INFO_to_keep, bool keep_all_INFO, bool stream)
 {
-	int N_kept = 0;
-	for (unsigned int ui=0; ui<include_entry.size(); ui++)
-		if (include_entry[ui] == true)
-			N_kept++;
-	return N_kept;
-}
+	LOG.printLOG("Outputting BCF file... ");
+	BGZF * out;
+	if(!stream)
+	{
+		string output_file = output_file_prefix + ".recode.bcf";
+		out = bgzf_open(output_file.c_str(), "w");
+	}
+	else
+		out = bgzf_dopen(1, "w");
 
-// Count the number of genotypes that have not been filtered out
-unsigned int vcf_file::N_genotypes_included(unsigned int entry_num) const
-{
-	unsigned int count = 0, ui;
-	for (ui=0; ui<N_indv; ui++)
-		if ((include_indv[ui] == true) && (include_genotype[entry_num][ui] == true))
-			count++;
+	print_bcf(out, INFO_to_keep, keep_all_INFO);
 
-	return count;
+	bgzf_close(out);
+	LOG.printLOG("Done\n");
 }
 
 void vcf_file::open()
 {
-	// Check if the file
-	struct stat buf ;
-	int i;
+	struct stat buf;
 
-	i = stat(filename.c_str(), &buf);
+	int i = stat(filename.c_str(), &buf);
 	if (i != 0)
+	{
+		perror("stat error");
 		LOG.error("Can't determine file type of " + filename, 0);
+	}
 	if (!S_ISREG(buf.st_mode))
 		LOG.error("Does not appear to be a regular file: " + filename, 0);
 
+	if (filename.substr(filename.size()-4) == ".bcf")
+		LOG.error("Filename ends in '.bcf'. Shouldn't you be using --bcf?\n");
+
 	if (!compressed)
 	{
 		if (filename.substr(filename.size()-3) == ".gz")
-			LOG.error("Filename ends in '.gz'. Shouldn't you be using --gzvcf?\n");
+			LOG.error("Filename ends in '.gz'. Shouldn't you be using --gzvcf or --gzdiff?\n");
 		vcf_in.open(filename.c_str(), ios::in);
 		if (!vcf_in.is_open())
 			LOG.error("Could not open VCF file: " + filename, 0);
@@ -374,7 +466,7 @@ void vcf_file::close()
 	}
 }
 
-bool vcf_file::feof()
+bool vcf_file::eof()
 {
 	bool out;
 	if (!compressed)
@@ -409,13 +501,17 @@ void vcf_file::set_filepos(streampos &filepos)
 	}
 }
 
-void vcf_file::get_vcf_entry(unsigned int entry_num, string &out)
+void vcf_file::get_entry(unsigned int entry_num, vector<char> &out)
 {
-	streampos filepos = entry_file_locations[entry_num];
-	set_filepos(filepos);
+	set_filepos( entry_file_locations[entry_num] );
 	read_line(out);
 }
 
+entry* vcf_file::get_entry_object(unsigned int N_indv)
+{
+	return new vcf_entry(N_indv);
+}
+
 void vcf_file::read_line(string &out)
 {
 	if (!compressed)
@@ -438,6 +534,32 @@ void vcf_file::read_line(string &out)
 	}
 }
 
+void vcf_file::read_line(vector<char> &out)
+{
+	static string tmp;
+	tmp.resize(0);
+	if (!compressed)
+	{
+		getline(vcf_in, tmp);
+		tmp.erase( tmp.find_last_not_of(" \t\n\r") + 1);	// Trim whitespace at end of line
+	}
+	else
+	{
+		bool again = true;
+		while (again == true)
+		{
+			gzgets(gzvcf_in, gz_readbuffer, gzMAX_LINE_LEN);
+			tmp.append(gz_readbuffer);
+			if (strlen(gz_readbuffer) != gzMAX_LINE_LEN-1)
+				again = false;
+		}
+		tmp.erase( tmp.find_last_not_of(" \t\n\r") + 1);	// Trim whitespace at end of line (required in gzipped case!)
+	}
+//	out.assign(tmp.begin(),tmp.end());
+	vector<char> tmp_char(tmp.begin(),tmp.end());
+	out = tmp_char;
+}
+
 char vcf_file::peek()
 {
 	if (!compressed)
@@ -450,8 +572,7 @@ char vcf_file::peek()
 	}
 }
 
-
-void vcf_file::read_CHROM_and_POS_and_skip_remainder_of_line(string &CHROM, int &POS)
+int vcf_file::read_CHROM_and_POS_and_skip_remainder_of_line(string &CHROM, int &POS)
 {
 	if (!compressed)
 	{
@@ -468,10 +589,9 @@ void vcf_file::read_CHROM_and_POS_and_skip_remainder_of_line(string &CHROM, int
 		getline(ss, CHROM, '\t');
 		ss >> POS;
 	}
+	return eof();
 }
 
-
-
 void vcf_file::read_CHROM_only(string &CHROM)
 {	// Just read in the chromosome. Note: leaves the stream in a funny state, but is faster than reading whole line
 	if (!compressed)
@@ -516,4 +636,3 @@ void vcf_file::read_CHROM_and_POS_only(string &CHROM, int &POS)
 		POS = atoi(tmp.c_str());
 	}
 }
-
